# This file is part of the Juju GUI, which lets users view and manage Juju
# environments within a graphical interface (https://launchpad.net/juju-gui).
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3, as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Juju Quickstart application management."""

from __future__ import print_function
import argparse
import os

import quickstart
from quickstart import (
    app,
    utils,
)


juju_home = os.getenv('JUJU_HOME', '~/.juju')
description = 'set up a Juju environment (including the GUI) in very few steps'
version = quickstart.get_version()


class _DescriptionAction(argparse.Action):
    """A customized argparse action that just shows a description."""

    def __call__(self, parser, *args, **kwargs):
        print(description)
        parser.exit()


def _validate_env(options, parser):
    """Validate and process the provided environment related options.

    Exit with an error if options are not valid.
    """
    env_file = os.path.abspath(os.path.expanduser(options.env_file))
    # Validate the environment name.
    env_name = options.env_name
    if env_name is None:
        return parser.error(
            'unable to find an environment name to use\n'
            'It is possible to specify the environment name by either:\n'
            '  - passing the -e or --environment argument;\n'
            '  - setting the JUJU_ENV environment variable;\n'
            '  - using "juju switch" to select the default environment to use.'
        )
    # Validate the environment file.
    try:
        env_type, admin_secret = utils.parse_env_file(env_file, env_name)
    except ValueError as err:
        return parser.error(str(err))
    # XXX 2013-10-15 frankban: add support for local providers.
    if env_type == 'local':
        return parser.error('the local provider is not currently supported')
    # Update the options namespace with the new values.
    options.admin_secret = admin_secret
    options.env_file = env_file
    options.env_type = env_type


def setup():
    """Set up the application options.

    Return the options as a namespace containing the followin attributes:
        - admin_secret: the password to use to access the Juju API;
        - env_file: the absolute path of the Juju environments.yaml file;
        - env_name: the name of the Juju environment to use;
        - env_type: the provider type of the selected Juju environment.

    Exit with an error if the provided arguments are not valid.
    """
    default_env_name = utils.get_default_env_name()
    # Define the help message for the --environment option.
    env_help = 'the name of the Juju environment to use'
    if default_env_name is not None:
        env_help = '{} (%(default)s)'.format(env_help)
    # Create and set up the arguments parser.
    parser = argparse.ArgumentParser(description=quickstart.__doc__)
    parser.add_argument(
        '-e', '--environment', default=default_env_name, dest='env_name',
        help=env_help)
    parser.add_argument(
        '--environments-file',
        default=os.path.join(juju_home, 'environments.yaml'), dest='env_file',
        help='the path to the Juju environments YAML file (%(default)s)')
    parser.add_argument(
        '--version', action='version', version='%(prog)s {}'.format(version))
    # This is required by juju-core: see "juju help plugins".
    parser.add_argument(
        '--description', action=_DescriptionAction, default=argparse.SUPPRESS,
        nargs=0, help="show program's description and exit")
    # Parse the provided arguments.
    options = parser.parse_args()
    # Validate and process the provided arguments.
    _validate_env(options, parser)
    return options


def run(options):
    """Run the application."""
    print('juju quickstart v{}'.format(version))
    print('bootstrapping the {} environment (type: {})'.format(
        options.env_name, options.env_type))
    app.bootstrap(options.env_name)
