# This file is part of the Juju GUI, which lets users view and manage Juju
# environments within a graphical interface (https://launchpad.net/juju-gui).
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3, as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Juju Quickstart base application functions."""

import json

from quickstart import utils


class ProgramExit(Exception):
    """An error occurred while setting up the Juju environment.

    Raise this exception if you want the program to exit gracefully printing
    the error message to stderr.
    """

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return 'juju-quickstart: error: {}'.format(self.message)


def bootstrap(env_name):
    """Bootstrap the Juju environment with the given name.

    Return when the bootstrap node is ready.
    Raise a ProgramExit if any error occurs in the bootstrap process.
    """
    retcode, _, error = utils.call('juju', 'bootstrap', '-e', env_name)
    if retcode:
        raise ProgramExit(error)
    # Call "juju status" multiple times until the bootstrap node is ready.
    for _ in range(5):
        retcode, output, error = utils.call(
            'juju', 'status', '-e', env_name, '--format', 'yaml')
        if retcode:
            continue
        # Ensure the state server is up and the agent is started.
        try:
            agent_state = utils.parse_status_output(output)
        except ValueError:
            continue
        if agent_state == 'started':
            return
    details = ''.join(filter(None, [output, error])).strip()
    raise ProgramExit('the state server is not ready:\n{}'.format(details))


def get_api_url(env_name):
    """Return a Juju API URL for the given environment name.

    Use the Juju CLI in a subprocess in order to retrieve the API addresses.
    Return the complete URL, e.g. "wss://api.example.com:17070".
    """
    retcode, output, error = utils.call(
        'juju', 'api-endpoints', '-e', env_name, '--format', 'json')
    if retcode:
        raise ProgramExit(error)
    # Assuming there is always at least one API address, grab the first one
    # from the JSON output.
    api_address = json.loads(output)[0]
    return 'wss://{}'.format(api_address)
