# This file is part of the Juju GUI, which lets users view and manage Juju
# environments within a graphical interface (https://launchpad.net/juju-gui).
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3, as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Juju Quickstart base application functions."""

from contextlib import contextmanager
import json
import unittest

import mock
import yaml

from quickstart import app
from quickstart.tests import helpers


class TestProgramExit(unittest.TestCase):

    def test_string_representation(self):
        # The error is properly represented as a string.
        exception = app.ProgramExit('bad wolf')
        self.assertEqual('juju-quickstart: error: bad wolf', str(exception))


class ProgramExitTestsMixin(object):
    """Set up some base methods for testing functions raising ProgramExit."""

    @contextmanager
    def assert_program_exit(self, error):
        """Ensure a ProgramExit is raised in the context block.

        Also check that the exception includes the expected error message.
        """
        with self.assertRaises(app.ProgramExit) as context_manager:
            yield
        expected = 'juju-quickstart: error: {}'.format(error)
        self.assertEqual(expected, str(context_manager.exception))


class TestBootstrap(
        helpers.CallTestsMixin, ProgramExitTestsMixin, unittest.TestCase):

    env_name = 'ec2'

    def make_status_output(self, agent_state):
        """Create and return a YAML status output."""
        return yaml.safe_dump({
            'machines': {'0': {'agent-state': agent_state}},
        })

    def make_status_calls(self, number):
        """Return a list containing the given number of status calls."""
        call = mock.call(
            'juju', 'status', '-e', self.env_name, '--format', 'yaml')
        return [call for _ in range(number)]

    def assert_status_retried(self, side_effects):
        """Ensure the "juju status" command is retried several times.

        Receive the list of side effects the mock status call will return.
        """
        with self.patch_multiple_calls(side_effects) as mock_call:
            app.bootstrap(self.env_name)
        mock_call.assert_has_calls([
            mock.call('juju', 'bootstrap', '-e', self.env_name),
        ] + self.make_status_calls(5))

    def test_success(self):
        # The environment is successfully bootstrapped.
        side_effects = [
            (0, '', ''),  # Add a bootstrap call.
            (0, self.make_status_output('started'), ''),  # Add a status call.
        ]
        with self.patch_multiple_calls(side_effects) as mock_call:
            app.bootstrap(self.env_name)
        mock_call.assert_has_calls([
            mock.call('juju', 'bootstrap', '-e', self.env_name),
        ] + self.make_status_calls(1))

    def test_bootstrap_failure(self):
        # A ProgramExit is raised if an error occurs while bootstrapping.
        with self.patch_call(1, error='bad wolf') as mock_call:
            with self.assert_program_exit('bad wolf'):
                app.bootstrap(self.env_name)
        mock_call.assert_called_once_with(
            'juju', 'bootstrap', '-e', self.env_name)

    def test_status_retry_error(self):
        # Before raising a ProgramExit, the functions tries to call
        # "juju status" multiple times if it exits with an error.
        side_effects = [
            (0, '', ''),  # Add the bootstrap call.
            # Add four status calls with a non-zero exit code.
            (1, '', 'these'),
            (2, '', 'are'),
            (3, '', 'the'),
            (4, '', 'voyages'),
            # Add a final valid status call.
            (0, self.make_status_output('started'), ''),
        ]
        self.assert_status_retried(side_effects)

    def test_status_retry_invalid_output(self):
        # Before raising a ProgramExit, the functions tries to call
        # "juju status" multiple times if its output is not well formed or if
        # the agent is not started.
        side_effects = [
            (0, '', ''),  # Add the bootstrap call.
            (0, '', ''),  # Add the first status call: no output.
            (0, ':', ''),  # Add the second status call: not YAML.
            (0, 'just-a-string', ''),  # Add the third status call: bad YAML.
            # Add the fourth status call: the agent is still pending.
            (0, self.make_status_output('pending'), ''),
            # Add a final valid status call.
            (0, self.make_status_output('started'), ''),
        ]
        self.assert_status_retried(side_effects)

    def test_status_retry_both(self):
        # Before raising a ProgramExit, the functions tries to call
        # "juju status" multiple times in any case.
        side_effects = [
            (0, '', ''),  # Add the bootstrap call.
            (1, '', 'error'),  # Add the first status call: error.
            (2, '', 'another error'),  # Add the second status call: error.
            # Add the third status call: the agent is still pending.
            (0, self.make_status_output('pending'), ''),
            (0, 'just-a-string', ''),  # Add the fourth status call: bad YAML.
            # Add a final valid status call.
            (0, self.make_status_output('started'), ''),
        ]
        self.assert_status_retried(side_effects)

    def test_status_failure(self):
        # A ProgramExit is raised if "juju status" keeps failing.
        status_side_effects = [
            (i, 'output #{}\n'.format(i), 'error #{}\n'.format(i))
            for i in range(5)
        ]
        side_effects = [(0, '', '')] + status_side_effects
        expected = 'the state server is not ready:\noutput #4\nerror #4'
        with self.patch_multiple_calls(side_effects) as mock_call:
            with self.assert_program_exit(expected):
                app.bootstrap(self.env_name)
        mock_call.assert_has_calls([
            mock.call('juju', 'bootstrap', '-e', self.env_name),
        ] + self.make_status_calls(5))


class TestGetApiUrl(
        helpers.CallTestsMixin, ProgramExitTestsMixin, unittest.TestCase):

    env_name = 'ec2'

    def test_success(self):
        # The API URL is correctly returned.
        api_addresses = json.dumps(['api.example.com:17070', 'not-today'])
        with self.patch_call(0, output=api_addresses) as mock_call:
            api_url = app.get_api_url(self.env_name)
        self.assertEqual('wss://api.example.com:17070', api_url)
        mock_call.assert_called_once_with(
            'juju', 'api-endpoints', '-e', self.env_name, '--format', 'json')

    def test_failure(self):
        # A ProgramExit is raised if an error occurs retrieving the API URL.
        with self.patch_call(1, error='bad wolf') as mock_call:
            with self.assert_program_exit('bad wolf'):
                app.get_api_url(self.env_name)
        mock_call.assert_called_once_with(
            'juju', 'api-endpoints', '-e', self.env_name, '--format', 'json')
