# This file is part of the Juju GUI, which lets users view and manage Juju
# environments within a graphical interface (https://launchpad.net/juju-gui).
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3, as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Juju Quickstart utility functions and classes."""

import unittest

import mock
import yaml

from quickstart import utils
from quickstart.tests import helpers


class TestCall(unittest.TestCase):

    def test_success(self):
        # A zero exit code and the subprocess output are correctly returned.
        retcode, output, error = utils.call('echo')
        self.assertEqual(0, retcode)
        self.assertEqual('\n', output)
        self.assertEqual('', error)

    def test_multiple_arguments(self):
        # A zero exit code and the subprocess output are correctly returned
        # when executing a command passing multiple arguments.
        retcode, output, error = utils.call('echo', 'we are the borg!')
        self.assertEqual(0, retcode)
        self.assertEqual('we are the borg!\n', output)
        self.assertEqual('', error)

    def test_failure(self):
        # An error code and the error are returned if the subprocess fails.
        retcode, output, error = utils.call('ls', 'no-such-file')
        self.assertNotEqual(0, retcode)
        self.assertEqual('', output)
        self.assertEqual(
            'ls: cannot access no-such-file: No such file or directory\n',
            error)

    def test_invalid_command(self):
        # An error code and the error are returned if the subprocess fails to
        # find the provided command in the PATH.
        retcode, output, error = utils.call('no-such-command')
        self.assertEqual(127, retcode)
        self.assertEqual('', output)
        self.assertEqual(
            'no-such-command: [Errno 2] No such file or directory',
            error)

    def test_logging(self):
        # The command line call and the results are properly logged.
        expected_messages = (
            "running the following: echo 'we are the borg!'",
            r"retcode: 0 | output: 'we are the borg!\n' | error: ''",
        )
        with helpers.assert_logs(expected_messages):
            utils.call('echo', 'we are the borg!')


class TestGetDefaultEnvName(helpers.CallTestsMixin, unittest.TestCase):

    def test_environment_variable(self):
        # The environment name is successfully returned if JUJU_ENV is set.
        with mock.patch('os.environ', {'JUJU_ENV': 'ec2'}):
            env_name = utils.get_default_env_name()
        self.assertEqual('ec2', env_name)

    def test_empty_environment_variable(self):
        # The environment name is not found if JUJU_ENV is empty.
        with self.patch_call(1):
            with mock.patch('os.environ', {'JUJU_ENV': ' '}):
                env_name = utils.get_default_env_name()
        self.assertIsNone(env_name)

    def test_no_environment_variable(self):
        # The environment name is not found if JUJU_ENV is not defined.
        with self.patch_call(1):
            with mock.patch('os.environ', {}):
                env_name = utils.get_default_env_name()
        self.assertIsNone(env_name)

    def test_juju_switch(self):
        # The environment name is successfully returned if previously set using
        # the "juju switch" command.
        output = 'Current environment: "hp"\n'
        with self.patch_call(0, output=output) as mock_call:
            with mock.patch('os.environ', {}):
                env_name = utils.get_default_env_name()
        self.assertEqual('hp', env_name)
        mock_call.assert_called_once_with('juju', 'switch')

    def test_juju_switch_failure(self):
        # The environment name is not found if "juju switch" returns an error.
        with self.patch_call(1) as mock_call:
            with mock.patch('os.environ', {}):
                env_name = utils.get_default_env_name()
        self.assertIsNone(env_name)
        mock_call.assert_called_once_with('juju', 'switch')

    def test_juju_switch_nonsense(self):
        # The environment name is not found if "juju switch" goes crazy.
        with self.patch_call(0, output='Exterminate!') as mock_call:
            with mock.patch('os.environ', {}):
                env_name = utils.get_default_env_name()
        self.assertIsNone(env_name)
        mock_call.assert_called_once_with('juju', 'switch')


class TestParseEnvFile(
        helpers.EnvFileTestsMixin, helpers.ValueErrorTestsMixin,
        unittest.TestCase):

    def test_no_file(self):
        # A ValueError is raised if the environments file is not found.
        expected = (
            'unable to open environments file: '
            "[Errno 2] No such file or directory: '/no/such/file.yaml'"
        )
        with self.assert_value_error(expected):
            utils.parse_env_file('/no/such/file.yaml', 'ec2')

    def test_invalid_yaml(self):
        # A ValueError is raised if the environments file is not a valid YAML.
        env_file = self.make_env_file(':')
        with self.assertRaises(ValueError) as context_manager:
            utils.parse_env_file(env_file, 'ec2')
        expected = 'unable to parse environments file {}'.format(env_file)
        self.assertIn(expected, str(context_manager.exception))

    def test_invalid_yaml_contents(self):
        # A ValueError is raised if the environments file is not well formed.
        env_file = self.make_env_file('a-string')
        expected = 'invalid YAML contents in {}: a-string'.format(env_file)
        with self.assert_value_error(expected):
            utils.parse_env_file(env_file, 'ec2')

    def test_no_env(self):
        # A ValueError is raised if the environment is not found in the YAML.
        contents = yaml.safe_dump({'environments': {'local': {}}})
        env_file = self.make_env_file(contents)
        expected = 'environment ec2 not found in {}'.format(env_file)
        with self.assert_value_error(expected):
            utils.parse_env_file(env_file, 'ec2')

    def test_no_provider_type(self):
        # A ValueError is raised if the provider type is not included in the
        # environment info.
        contents = yaml.safe_dump({'environments': {'aws': {}}})
        env_file = self.make_env_file(contents)
        expected = 'aws provider type not found in {}'.format(env_file)
        with self.assert_value_error(expected):
            utils.parse_env_file(env_file, 'aws')

    def test_no_admin_secret(self):
        # A ValueError is raised if the admin secret is not included in the
        # environment info.
        contents = yaml.safe_dump({'environments': {'aws': {'type': 'ec2'}}})
        env_file = self.make_env_file(contents)
        expected = 'aws admin secret not found in {}'.format(env_file)
        with self.assert_value_error(expected):
            utils.parse_env_file(env_file, 'aws')

    def test_success(self):
        # The environment provider type and admin secret are returned.
        env_file = self.make_env_file()
        env_type, admin_secret = utils.parse_env_file(env_file, 'aws')
        self.assertEqual('ec2', env_type)
        self.assertEqual('Secret!', admin_secret)


class TestParseStatusOutput(helpers.ValueErrorTestsMixin, unittest.TestCase):

    def test_invalid_yaml(self):
        # A ValueError is raised if the output is not a valid YAML.
        with self.assertRaises(ValueError) as context_manager:
            utils.parse_status_output(':')
        expected = 'unable to parse the output'
        self.assertIn(expected, str(context_manager.exception))

    def test_invalid_yaml_contents(self):
        # A ValueError is raised if the output is not well formed.
        with self.assert_value_error('invalid YAML contents: a-string'):
            utils.parse_status_output('a-string')

    def test_no_env(self):
        # A ValueError is raised if the agent-state is not found in the YAML.
        data = {
            'machines': {
                '0': {'agent-version': '1.17.0.1'},
            },
        }
        expected = 'agent state not found in {}'.format(str(data))
        with self.assert_value_error(expected):
            utils.parse_status_output(yaml.safe_dump(data))

    def test_success(self):
        # The agent state is correctly returned.
        output = yaml.safe_dump({
            'machines': {
                '0': {'agent-version': '1.17.0.1', 'agent-state': 'started'},
            },
        })
        agent_state = utils.parse_status_output(output)
        self.assertEqual('started', agent_state)
