# This file is part of the Juju GUI, which lets users view and manage Juju
# environments within a graphical interface (https://launchpad.net/juju-gui).
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License version 3, as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Juju Quickstart utility functions and classes."""

import re
import os
import logging
import pipes
import subprocess

import yaml

# Compile the regular expression used to parse the "juju switch" output.
_juju_switch_expression = re.compile(r'Current environment: "([\w-]+)"\n')


def call(*args):
    """Call a subprocess passing the given arguments.

    Take the subcommand and its parameters as args.

    Return a tuple containing the subprocess return code, output and error.
    """
    pipe = subprocess.PIPE
    cmdline = ' '.join(map(pipes.quote, args))
    logging.debug('running the following: {}'.format(cmdline))
    try:
        process = subprocess.Popen(args, stdout=pipe, stderr=pipe)
    except OSError as err:
        # A return code 127 is returned by the shell when the command is not
        # found in the PATH.
        return 127, '', '{}: {}'.format(args[0], err)
    output, error = process.communicate()
    retcode = process.poll()
    logging.debug('retcode: {} | output: {!r} | error: {!r}'.format(
        retcode, output, error))
    return retcode, output, error


def get_default_env_name():
    """Return the current Juju environment name.

    The environment name can be set either by
        - setting the JUJU_ENV environment variable;
        - using "juju switch my-env-name";
        - setting the default environment in the environments.yaml file.
    The former overrides the latter.

    Return None if a default environment is not found.
    """
    env_name = os.getenv('JUJU_ENV', '').strip()
    if env_name:
        return env_name
    # XXX 2013-10-16 frankban bug=1193244:
        # Switch to using "juju switch --raw" in order to avoid the fragility
        # of parsing the command output.
    # The "juju switch" command parses ~/.juju/current-environment file. If the
    # environment name is not found there, then it tries to retrieve the name
    # from the "default" section of the ~/.juju/environments.yaml file.
    retcode, output, _ = call('juju', 'switch')
    if retcode:
        return None
    match = _juju_switch_expression.match(output)
    if match is not None:
        return match.groups()[0]


def parse_env_file(env_file, env_name):
    """Parse the provided Juju environments.yaml file.

    Return a tuple containing the provider type and the admin secret associated
    with the given environment name.

    Raise a ValueError if:
      - the environment file is not found;
      - the environment file contents are not parsable by YAML;
      - the YAML contents are not properly structured;
      - the environment named envname is not found;
      - the environment does not include the "type" field;
      - the environment does not include the "admin_secret" field.
    """
    # Load the Juju environments file.
    try:
        environments_file = open(env_file)
    except IOError as err:
        msg = 'unable to open environments file: {}'.format(err)
        raise ValueError(msg)
    # Parse the Juju environments file.
    try:
        environments = yaml.safe_load(environments_file)
    except Exception as err:
        msg = 'unable to parse environments file {}: {}'.format(env_file, err)
        raise ValueError(msg)
    # Retrieve the information about the current environment.
    try:
        environment = environments.get('environments', {}).get(env_name)
    except AttributeError as err:
        msg = 'invalid YAML contents in {}: {}'.format(env_file, environments)
        raise ValueError(msg)
    if environment is None:
        msg = 'environment {} not found in {}'.format(env_name, env_file)
        raise ValueError(msg)
    # Retrieve the provider type and the admin secret.
    env_type = environment.get('type')
    if env_type is None:
        msg = '{} provider type not found in {}'.format(env_name, env_file)
        raise ValueError(msg)
    admin_secret = environment.get('admin-secret')
    if admin_secret is None:
        msg = '{} admin secret not found in {}'.format(env_name, env_file)
        raise ValueError(msg)
    return env_type, admin_secret


def parse_status_output(output):
    """Parse the output of juju status.

    Return the agent state.
    Raise a ValueError if the agent state cannot be retrieved.
    """
    try:
        status = yaml.safe_load(output)
    except Exception as err:
        raise ValueError('unable to parse the output: {}'.format(err))
    try:
        state = status.get('machines', {}).get('0', {}).get('agent-state')
    except AttributeError as err:
        raise ValueError('invalid YAML contents: {}'.format(status))
    if state is None:
        raise ValueError('agent state not found in {}'.format(status))
    return state
